@@ -1,5 +1,6 @@
 import csv
 import io
+import itertools
 import json
 import sys
 
@@ -24,6 +25,7 @@ def _get_query_data(bound_query):
         "filters": [
             {
                 "path": filter_.path,
+                "pathStr": filter_.path_str,
                 "prettyPath": filter_.pretty_path,
                 "lookup": filter_.lookup,
                 "value": filter_.value,
@@ -34,9 +36,11 @@ def _get_query_data(bound_query):
         "fields": [
             {
                 "path": field.path,
+                "pathStr": field.path_str,
                 "prettyPath": field.pretty_path,
                 "sort": field.direction,
                 "priority": field.priority,
+                "pivoted": field.pivoted,
             }
             for field in bound_query.fields
         ],
@@ -56,6 +60,7 @@ def _get_model_fields(orm_model):
             "model": orm_field.rel_name,
             "type": orm_field.type_.name if orm_field.type_ else None,
             "concrete": orm_field.concrete,
+            "canPivot": orm_field.can_pivot,
             "prettyName": orm_field.pretty_name,
         }
         for name, orm_field in orm_model.fields.items()
@@ -86,7 +91,7 @@ def _get_config(user, orm_models):
             "public": view.public,
             "model": view.model_name,
             "description": view.description,
-            "query": _get_query_data(BoundQuery(view.get_query(), orm_models)),
+            "query": _get_query_data(BoundQuery.bind(view.get_query(), orm_models)),
         }
         for view in View.objects.filter(owner=user).order_by("name")
     ]
@@ -113,10 +118,15 @@ def _get_context(request, model_name, fields):
     orm_models = get_models(request)
     if query.model_name and query.model_name not in orm_models:
         raise http.Http404(f"{query.model_name} does not exist")
-    bound_query = BoundQuery(query, orm_models)
+    bound_query = BoundQuery.bind(query, orm_models)
     return {
         "config": _get_config(request.user, orm_models),
-        "initialState": {"results": [], **_get_query_data(bound_query)},
+        "initialState": {
+            "results": [],
+            "cols": [],
+            "rows": [],
+            **_get_query_data(bound_query),
+        },
         "sentryDsn": getattr(settings, "DATA_BROWSER_FE_DSN", None),
     }
 
@@ -153,7 +163,7 @@ def query(request, *, model_name, fields="", media):
 
 
 def view(request, pk, media):
-    view = get_object_or_404(View.objects.filter(public=True), pk=pk)
+    view = get_object_or_404(View.objects.filter(public=True), public_slug=pk)
     if (
         # some of these are checked by the admin but this is a good time to be paranoid
         view.owner.is_active
@@ -168,30 +178,87 @@ def view(request, pk, media):
         raise http.Http404("No View matches the given query.")
 
 
+def pad(x):
+    return [None] * max(0, x)
+
+
+def concat(*lists):
+    return list(itertools.chain.from_iterable(lists))
+
+
+def flip_table(table):
+    return [list(x) for x in zip(*table)]
+
+
+def join_tables(*tables):
+    return [concat(*ts) for ts in zip(*tables)]
+
+
+def format_table(fields, table, spacing=0):
+    return concat(
+        [[" ".join(f.pretty_path) for f in fields]],
+        *[
+            [[row[f.path_str] for f in fields]] + [pad(len(fields))] * spacing
+            for row in table
+        ],
+    )
+
+
+def pad_table(x, table):
+    return [pad(x) + row for row in table]
+
+
 def _data_response(request, query, media, meta):
     orm_models = get_models(request)
     if query.model_name not in orm_models:
         raise http.Http404(f"{query.model_name} does not exist")
-    bound_query = BoundQuery(query, orm_models)
-    results = get_results(request, bound_query)
+    bound_query = BoundQuery.bind(query, orm_models)
+    results = get_results(request, bound_query, orm_models)
 
     if media == "csv":
         buffer = io.StringIO()
         writer = csv.writer(buffer)
-        writer.writerow(" ".join(f.pretty_path) for f in bound_query.fields)
-        writer.writerows(results)
+
+        # the pivoted column headers
+        writer.writerows(
+            pad_table(
+                len(bound_query.row_fields) - 1,
+                flip_table(
+                    format_table(
+                        bound_query.col_fields,
+                        results["cols"],
+                        spacing=len(bound_query.data_fields) - 1,
+                    )
+                ),
+            )
+        )
+
+        # the row headers and data area
+        writer.writerows(
+            pad_table(
+                1 - len(bound_query.row_fields),
+                join_tables(
+                    format_table(bound_query.row_fields, results["rows"]),
+                    *(
+                        format_table(bound_query.data_fields, sub_table)
+                        for sub_table in results["body"]
+                    ),
+                ),
+            )
+        )
+
         buffer.seek(0)
-        response = http.HttpResponse(buffer, content_type="text/csv")
+        response = http.HttpResponse(buffer, content_type="text")
         response[
             "Content-Disposition"
         ] = f"attachment; filename={query.model_name}-{timezone.now().isoformat()}.csv"
         return response
     elif media == "json":
         resp = _get_query_data(bound_query) if meta else {}
-        resp["results"] = results
+        resp.update(results)
         return http.JsonResponse(resp)
     else:
-        assert False
+        raise http.Http404(f"Bad file format {media} requested")
 
 
 def _get_from_js_dev_server(request):  # pragma: no cover
