@@ -16,6 +16,7 @@ from data_browser.query import (
     QueryFilter,
     StringType,
     WeekDayType,
+    YearType,
 )
 from django.http import QueryDict
 from django.utils import timezone
@@ -285,6 +286,22 @@ class TestNumberType:
         assert NumberType.format(6) == 6
 
 
+class TestYearType:
+    def test_validate(self):
+        assert YearType.parse("gt", "6") == (6, None)
+        assert YearType.parse("pontains", "6.1") == (None, ANY(str))
+        assert YearType.parse("gt", "hello") == (None, ANY(str))
+        assert YearType.parse("is_null", "True") == (True, None)
+        assert YearType.parse("is_null", "hello") == (None, ANY(str))
+        assert YearType.parse("equals", "0") == (None, ANY(str))
+
+    def test_default_lookup(self):
+        assert YearType.default_lookup == "equals"
+
+    def test_format(self):
+        assert YearType.format(6) == 6
+
+
 class TestDateTimeType:
     def test_validate(self):
         assert DateTimeType.parse("gt", "2018-03-20T22:31:23") == (ANY(datetime), None)
