@@ -1,15 +1,13 @@
 import csv
 import json
+from datetime import datetime
 
 import data_browser.models
 import pytest
 from django.contrib.auth.models import User
 
 from . import models
-
-true = True
-false = False
-null = None
+from .util import update_fe_fixture
 
 
 def dump(val):
@@ -33,6 +31,22 @@ def products(db):
     models.Product.objects.create(name="c", size=2, size_unit="g", producer=producer)
 
 
+@pytest.fixture
+def pivot_products(db):
+    address = models.Address.objects.create(city="london", street="bad")
+    producer = models.Producer.objects.create(name="Bob", address=address)
+    datetimes = [
+        datetime(2020, 1, 1),
+        datetime(2020, 2, 1),
+        datetime(2020, 2, 2),
+        datetime(2021, 1, 1),
+        datetime(2021, 1, 2),
+        datetime(2021, 1, 3),
+    ]
+    for dt in datetimes:
+        models.Product.objects.create(created_time=dt, name=str(dt), producer=producer)
+
+
 def test_query_html(admin_client, snapshot):
     res = admin_client.get(
         "/data_browser/query/tests.Product/size-0,name+1,size_unit.html?size__lt=2&id__gt=0"
@@ -62,9 +76,7 @@ def test_query_ctx(admin_client, snapshot):
     context["config"]["version"] = "redacted"
     context["initialState"]["version"] = "redacted"
     snapshot.assert_match(context, "context")
-    with open("frontend/src/context_fixture.json", "w") as f:
-        json.dump(context, f, indent=4, sort_keys=True)
-        f.write("\n")
+    update_fe_fixture("frontend/src/context_fixture.json", context)
 
 
 @pytest.mark.usefixtures("products")
@@ -79,9 +91,21 @@ def test_query_json_bad_fields(admin_client):
         )
     )
     assert res.status_code == 200
-    assert json.loads(res.content.decode("utf-8"))["results"] == [
-        [1, "a", "g", False, "Bob"],
-        [1, "b", "g", False, "Bob"],
+    assert json.loads(res.content.decode("utf-8"))["rows"] == [
+        {
+            "size": 1,
+            "name": "a",
+            "size_unit": "g",
+            "is_onsale": False,
+            "producer__name": "Bob",
+        },
+        {
+            "size": 1,
+            "name": "b",
+            "size_unit": "g",
+            "is_onsale": False,
+            "producer__name": "Bob",
+        },
     ]
 
 
@@ -92,6 +116,13 @@ def test_query_html_bad_model(admin_client):
     assert res.status_code == 404
 
 
+def test_query_html_bad_media(admin_client):
+    res = admin_client.get(
+        "/data_browser/query/tests.Product/size-0,name+1,size_unit.bob?size__lt=2&id__gt=0"
+    )
+    assert res.status_code == 404
+
+
 @pytest.mark.usefixtures("products")
 def test_query_csv(admin_client):
     res = admin_client.get(
@@ -104,6 +135,65 @@ def test_query_csv(admin_client):
     assert rows == [["size", "name", "size_unit"], ["1.0", "a", "g"], ["1.0", "b", "g"]]
 
 
+@pytest.mark.usefixtures("pivot_products")
+def test_query_csv_pivoted(admin_client):
+    res = admin_client.get(
+        "/data_browser/query/tests.Product/created_time__year,&created_time__month,id__count,id__max.csv?"
+    )
+    assert res.status_code == 200
+    print(res.content.decode("utf-8"))
+    rows = list(csv.reader(res.content.decode("utf-8").splitlines()))
+    dump(rows)
+    assert rows == [
+        ["created_time month", "January", "", "Feburary", ""],
+        ["created_time year", "id count", "id max", "id count", "id max"],
+        ["2020.0", "1.0", "1.0", "2.0", "3.0"],
+        ["2021.0", "3.0", "6.0", "", ""],
+    ]
+
+
+testdata = [
+    ("----"),
+    ("---b"),
+    ("--c-"),
+    ("--cb"),
+    ("-r--"),
+    ("-r-b"),
+    ("-rc-"),
+    ("-rcb"),
+    ("d---"),
+    ("d--b"),
+    ("d-c-"),
+    ("d-cb"),
+    ("dr--"),
+    ("dr-b"),
+    ("drc-"),
+    ("drcb"),
+]
+
+
+@pytest.mark.usefixtures("pivot_products")
+@pytest.mark.parametrize("key", testdata)
+def test_query_csv_pivot_permutations(admin_client, key, snapshot):
+    fields = []
+    if "r" in key:
+        fields.append("created_time__year")
+    if "c" in key:
+        fields.append("&created_time__month")
+    if "b" in key:
+        fields.extend(["id__count", "id__max"])
+    filters = "" if "d" in key else "id__equals=123"
+
+    res = admin_client.get(
+        f"/data_browser/query/tests.Product/{','.join(fields)}.csv?{filters}"
+    )
+    assert res.status_code == 200
+    print(res.content.decode("utf-8"))
+    rows = list(csv.reader(res.content.decode("utf-8").splitlines()))
+    dump(rows)
+    snapshot.assert_match(rows, "key")
+
+
 @pytest.mark.usefixtures("products")
 def test_query_json(admin_client, snapshot):
     res = admin_client.get(
@@ -115,6 +205,17 @@ def test_query_json(admin_client, snapshot):
     snapshot.assert_match(data, "data")
 
 
+@pytest.mark.usefixtures("pivot_products")
+def test_query_json_pivot(admin_client, snapshot):
+    res = admin_client.get(
+        "/data_browser/query/tests.Product/created_time__year,&created_time__month,id__count,id__max.json?"
+    )
+    assert res.status_code == 200
+    data = json.loads(res.content.decode("utf-8"))
+    data["version"] = "redacted"
+    snapshot.assert_match(data, "data")
+
+
 @pytest.mark.usefixtures("products")
 def test_query_json_bad_model(admin_client):
     res = admin_client.get(
@@ -132,12 +233,12 @@ def test_view_csv(admin_client, settings):
         owner=User.objects.get(),
     )
 
-    res = admin_client.get(f"/data_browser/view/{view.pk}.csv")
+    res = admin_client.get(f"/data_browser/view/{view.public_slug}.csv")
     assert res.status_code == 404
 
     view.public = True
     view.save()
-    res = admin_client.get(f"/data_browser/view/{view.pk}.csv")
+    res = admin_client.get(f"/data_browser/view/{view.public_slug}.csv")
     assert res.status_code == 200
     print(res.content.decode("utf-8"))
     rows = list(csv.reader(res.content.decode("utf-8").splitlines()))
@@ -145,13 +246,13 @@ def test_view_csv(admin_client, settings):
     assert rows == [["size", "name", "size_unit"], ["1.0", "a", "g"], ["1.0", "b", "g"]]
 
     settings.DATA_BROWSER_ALLOW_PUBLIC = False
-    res = admin_client.get(f"/data_browser/view/{view.pk}.csv")
+    res = admin_client.get(f"/data_browser/view/{view.public_slug}.csv")
     assert res.status_code == 404
     settings.DATA_BROWSER_ALLOW_PUBLIC = True
 
     view.owner = User.objects.create(is_staff=True)
     view.save()
-    res = admin_client.get(f"/data_browser/view/{view.pk}.csv")
+    res = admin_client.get(f"/data_browser/view/{view.public_slug}.csv")
     assert res.status_code == 404
 
 
@@ -164,18 +265,25 @@ def test_view_json(admin_client):
         owner=User.objects.get(),
     )
 
-    res = admin_client.get(f"/data_browser/view/{view.pk}.json")
+    res = admin_client.get(f"/data_browser/view/{view.public_slug}.json")
     assert res.status_code == 404
 
     view.public = True
     view.save()
-    res = admin_client.get(f"/data_browser/view/{view.pk}.json")
+    res = admin_client.get(f"/data_browser/view/{view.public_slug}.json")
     assert res.status_code == 200
     data = json.loads(res.content.decode("utf-8"))
     dump(data)
-    assert data == {"results": [[1, "a", "g"], [1, "b", "g"]]}
+    assert data == {
+        "rows": [
+            {"size": 1, "name": "a", "size_unit": "g"},
+            {"size": 1, "name": "b", "size_unit": "g"},
+        ],
+        "cols": [{}],
+        "body": [[{}, {}]],
+    }
 
     view.owner = User.objects.create(is_staff=True)
     view.save()
-    res = admin_client.get(f"/data_browser/view/{view.pk}.csv")
+    res = admin_client.get(f"/data_browser/view/{view.public_slug}.csv")
     assert res.status_code == 404
